import React, { forwardRef, useRef, useCallback, useMemo, useImperativeHandle } from 'react';
import classnames from 'classnames';
import { addUnit } from '../utils/index.js';
import Icon from '../icon/index.js';

const Checker = /*#__PURE__*/forwardRef((props, ref) => {
  const {
    shape = 'round',
    imgUrl,
    disabled = false,
    labelDisabled = false,
    labelPosition = 'right',
    iconSize = '16px',
    bindGroup = true,
    checked,
    checkedColor = '#1989fa',
    parent,
    role,
    onClick,
    onToggle,
    children,
    baseClass
  } = props;
  const iconRef = useRef(null);
  const getParentProp = useCallback(arg => {
    if (parent && bindGroup) {
      return parent[arg];
    }

    return null;
  }, [parent, bindGroup]); // 父组件传递的排列方向

  const directionVal = () => {
    return getParentProp('direction') || null;
  }; // 是否禁用


  const disabledStatus = useMemo(() => {
    return getParentProp('disabled') || disabled;
  }, [getParentProp, disabled]); // icon行内样式

  const iconStyle = useCallback(() => {
    const checkedStatusColor = getParentProp('checkedColor') || checkedColor;

    if (checked && checkedStatusColor && !disabledStatus) {
      return {
        borderColor: checkedStatusColor,
        backgroundColor: checkedStatusColor
      };
    }

    return null;
  }, [checkedColor, checked, disabledStatus, getParentProp]); // 外层样式

  const iconWrapperClassString = classnames(`${baseClass}__icon`, {
    [`${baseClass}__icon--round`]: shape === 'round',
    [`${baseClass}__icon--disabled`]: disabledStatus,
    [`${baseClass}__icon--checked`]: checked,
    [`${baseClass}__icon--double`]: disabledStatus && checked
  });
  const classString = classnames(baseClass, `${baseClass}--${directionVal()}`, {
    [`${baseClass}--label-disabled`]: labelDisabled,
    [`${baseClass}--disabled`]: disabledStatus
  });
  const renderIcon = useMemo(() => {
    return /*#__PURE__*/React.createElement("div", {
      ref: iconRef,
      className: iconWrapperClassString,
      style: {
        fontSize: addUnit((parent == null ? void 0 : parent.iconSize) || iconSize)
      }
    }, imgUrl ? /*#__PURE__*/React.createElement("img", {
      alt: "pic",
      src: checked ? imgUrl.activeIcon : imgUrl.inactiveIcon,
      className: classnames(`${baseClass}__imgUrl`)
    }) : /*#__PURE__*/React.createElement(Icon, {
      name: "success",
      className: classnames('r-icon'),
      style: iconStyle()
    }));
  }, [iconSize, parent == null ? void 0 : parent.iconSize, checked, iconWrapperClassString, imgUrl, baseClass, iconStyle]);
  const renderLabel = useMemo(() => {
    const classString = classnames(`${baseClass}__label`, `${baseClass}__label--${labelPosition}`, {
      [`${baseClass}__label--disabled`]: disabledStatus
    });
    return /*#__PURE__*/React.createElement(React.Fragment, null, children && /*#__PURE__*/React.createElement("span", {
      className: classString
    }, children));
  }, [labelPosition, disabledStatus, children, baseClass]);
  const nodes = [renderIcon];

  if (labelPosition === 'left') {
    nodes.unshift(renderLabel);
  } else {
    nodes.push(renderLabel);
  } // 点击checkbox


  const handleClick = e => {
    const {
      target
    } = e;
    const icon = iconRef.current; // 当点击的是复选框 或者 点击的区域包括复选框

    const iconClicked = icon === target || (icon == null ? void 0 : icon.contains(target)); // 复选框没被禁用，并且（复选框被点击或者文本内容没被禁用的情况下）

    if (!disabledStatus && (iconClicked || !labelDisabled)) {
      // 状态取反
      onToggle == null ? void 0 : onToggle(!checked);
    } // 触发onClick事件


    onClick == null ? void 0 : onClick(e);
  };

  useImperativeHandle(ref, () => ({
    handleClick
  }));
  return /*#__PURE__*/React.createElement("div", {
    ref: ref,
    onClick: handleClick,
    className: classString,
    role: role,
    tabIndex: disabledStatus ? -1 : 0,
    "aria-checked": checked
  }, nodes.map(item => {
    return /*#__PURE__*/React.cloneElement(item, {
      key: item == null ? void 0 : item.toString()
    });
  }));
});
Checker.displayName = 'Checker';

export default Checker;
