import React, { useState, useEffect, useCallback, useMemo } from 'react';
import classnames from 'classnames';
import { addUnit } from '../utils/index.js';
import Popup from '../popup/index.js';
import Button from '../button/index.js';
import ActionBar from '../action-bar/index.js';
import { callIntercepter } from '../utils/callIntercepter.js';

const baseClass = 'r-dialog';

const Dialog = props => {
  const {
    show = false,
    title,
    theme = 'default',
    width = '320px',
    message = '',
    className,
    onBeforeClose,
    messageAlign = 'center',
    showCancelButton = false,
    cancelButtonText = '取消',
    cancelButtonColor = '#000000',
    confirmButtonText = '确认',
    confirmButtonColor = '#ee0a24',
    showConfirmButton = true,
    onClickOverlay,
    onConfirm,
    onCancel,
    onOpen,
    onOpened,
    onClose,
    onClosed,
    updateShow
  } = props;
  const [visible, setVisible] = useState(false);
  const [loadingConfirm, setLoadingConfirm] = useState(false);
  const [loadingCancel, setLoadingCancel] = useState(false);
  useEffect(() => {
    setVisible(show);
  }, [show]);

  const handleLoadingStatus = (action, status) => {
    // eslint-disable-next-line no-param-reassign
    // action === 'confirm' ? setLoadingConfirm(status) : setLoadingCancel(status);
    if (action === 'confirm') {
      setLoadingConfirm(status);
    } else {
      setLoadingCancel(status);
    }
  }; // 处理按钮的点击事件


  const handleAction = useCallback(action => {
    if (!show) {
      return;
    }

    if (action === 'confirm') {
      onConfirm == null ? void 0 : onConfirm();
    } else {
      onCancel == null ? void 0 : onCancel();
    }

    if (onBeforeClose) {
      handleLoadingStatus(action, true);
      callIntercepter({
        interceptor: onBeforeClose,
        args: [action],
        done: () => {
          handleLoadingStatus(action, false);
          updateShow == null ? void 0 : updateShow(false);
        },
        canceled: () => {
          handleLoadingStatus(action, false);
          updateShow == null ? void 0 : updateShow(false);
        }
      });
    } else {
      updateShow == null ? void 0 : updateShow(false);
    }
  }, [show, onBeforeClose, onConfirm, onCancel, updateShow]);
  const renderTitle = useMemo(() => {
    const titleCls = classnames(`${baseClass}__header`, {
      [`${baseClass}__header--isolated`]: !message
    });

    if (title) {
      return /*#__PURE__*/React.createElement("div", {
        className: titleCls
      }, title);
    }

    return null;
  }, [title, message]);
  const renderContent = useMemo(() => {
    const classStringWrapper = classnames(`${baseClass}__content`, {
      [`${baseClass}__content--isolated`]: !title
    });
    const classStringInner = classnames(`${baseClass}__message`, {
      [`${baseClass}__message--has-title`]: title,
      [`${baseClass}__message--${messageAlign}`]: messageAlign
    });

    if (message) {
      return /*#__PURE__*/React.createElement("div", {
        className: classStringWrapper
      }, /*#__PURE__*/React.createElement("div", {
        className: classStringInner
      }, message));
    }

    return null;
  }, [message, title, messageAlign]);
  const renderButtons = useMemo(() => {
    return /*#__PURE__*/React.createElement("div", {
      className: classnames(`${baseClass}__footer`, `${baseClass}__border--top`)
    }, showCancelButton && /*#__PURE__*/React.createElement(Button, {
      size: "large",
      text: cancelButtonText,
      className: classnames(`${baseClass}__cancel`),
      style: {
        color: cancelButtonColor
      },
      loading: loadingCancel,
      onClick: () => handleAction('cancel')
    }), showConfirmButton && /*#__PURE__*/React.createElement(Button, {
      size: "large",
      text: confirmButtonText,
      className: classnames(`${baseClass}__confirm`, {
        [`${baseClass}__border--left`]: showCancelButton
      }),
      style: {
        color: confirmButtonColor
      },
      loading: loadingConfirm,
      onClick: () => handleAction('confirm')
    }));
  }, [showCancelButton, loadingCancel, cancelButtonText, cancelButtonColor, showConfirmButton, loadingConfirm, confirmButtonText, confirmButtonColor, handleAction]);
  const renderRoundButtons = useMemo(() => {
    return /*#__PURE__*/React.createElement(ActionBar, {
      className: classnames(`${baseClass}__footer`)
    }, showCancelButton && /*#__PURE__*/React.createElement(Button, {
      size: "large",
      type: "warning",
      text: cancelButtonText,
      className: classnames(`${baseClass}__cancel`, `${baseClass}__cancel__space-around`),
      round: true,
      color: cancelButtonColor,
      loading: loadingCancel,
      onClick: () => handleAction('cancel')
    }), showConfirmButton && /*#__PURE__*/React.createElement(Button, {
      size: "large",
      type: "danger",
      text: confirmButtonText,
      round: true,
      className: classnames(`${baseClass}__confirm`, `${baseClass}__confirm__space-around`),
      color: confirmButtonColor,
      loading: loadingConfirm,
      onClick: () => handleAction('confirm')
    }));
  }, [showCancelButton, loadingCancel, cancelButtonText, cancelButtonColor, showConfirmButton, loadingConfirm, confirmButtonText, confirmButtonColor, handleAction]);
  return /*#__PURE__*/React.createElement(Popup, {
    show: visible,
    className: classnames(baseClass, {
      [`${baseClass}--${theme}`]: !!theme
    }, className),
    style: {
      width: addUnit(width)
    },
    onClickOverlay: onClickOverlay,
    onOpen: onOpen,
    onOpened: onOpened,
    onClose: onClose,
    onClosed: onClosed
  }, renderTitle, renderContent, theme === 'round-button' ? renderRoundButtons : renderButtons);
};

export default Dialog;
