import _extends from '@babel/runtime/helpers/esm/extends';
import React, { useRef, useCallback, forwardRef, useState, useContext, useMemo, useImperativeHandle, useEffect } from 'react';
import classnames from 'classnames';
import Cell from '../cell/index.js';
import Icon from '../icon/index.js';
import { FormContenxt } from '../form/index.js';
import { resetScroll } from '../utils/scroll.js';
import { addUnit, isDef, runSyncRule, isPromise, isFunction, trigger, isObject, formatNumber } from '../utils/index.js';

const baseClass = 'r-field';
const useRefCallback = callback => {
  const callbackRef = useRef(callback);
  callbackRef.current = callback;
  return useCallback((...args) => callbackRef.current(...args), []);
};
const Field = /*#__PURE__*/forwardRef((props, ref) => {
  const {
    model,
    name,
    rules,
    type = 'text',
    maxlength,
    clearable = false,
    clearTrigger = 'focus',
    formatter,
    formatTrigger = 'onChange',
    error = false,
    rows,
    autosize = false,
    leftIcon,
    rightIcon,
    iconPrefix = 'r-icon',
    showWordLimit = false,
    errorMessage,
    label,
    center = false,
    border = true,
    isLink = false,
    required = false,
    clickable = false,
    labelClass,
    arrowDirection = 'right',
    size,
    disabled = false,
    labelAlign = 'left',
    placeholder,
    autocomplete,
    autofocus = false,
    onFocus,
    onBlur,
    onKeyPress,
    onClickInput,
    onUpdateModel,
    onClickRightIcon,
    onClear,
    children
  } = props;
  const [modelValue, setModelValue] = useState(model);
  const [focused, setFocused] = useState(false);
  const [validateFailed, setValidateFailed] = useState(false);
  const [validateMessage, setValidateMessage] = useState('');
  const [composing, setComposing] = useState(false);
  const inputRef = useRef(null);
  const childFieldValue = useRef(null);
  const parent = useContext(FormContenxt);

  const getModelValue = () => String(modelValue != null ? modelValue : '');

  const getProp = key => {
    if (isDef(props[key])) {
      return props[key];
    }

    if (parent && isDef(parent[key])) {
      return parent[key];
    }

    return null;
  };

  const showClear = useMemo(() => {
    const readonly = getProp('readonly');

    if (clearable && !readonly) {
      const hasValue = getModelValue() !== '';
      const trigger = clearTrigger === 'always' || clearTrigger === 'focus' && focused;
      return hasValue && trigger;
    }

    return null;
  }, [clearable, clearTrigger, focused, getProp, getModelValue]);

  const runValidator = (value, rule) => {
    // eslint-disable-next-line consistent-return
    new Promise(resolve => {
      const returnVal = rule.validator == null ? void 0 : rule.validator(value, rule);

      if (isPromise(returnVal)) {
        return returnVal.then(resolve);
      }

      resolve(returnVal);
    });
  };

  const getRuleMessage = (value, rule) => {
    const {
      message
    } = rule;

    if (isFunction(message)) {
      return message(value, rule);
    }

    return message || '';
  };

  const runRules = rules => rules.reduce((promise, rule) => promise.then(() => {
    if (validateFailed) {
      return;
    } // let { current } = formValue;


    let {
      current
    } = childFieldValue;

    if (rule.formatter) {
      current = rule.formatter(current, rule);
    }

    if (!runSyncRule(current, rule)) {
      setValidateFailed(true);
      setValidateMessage(getRuleMessage(current, rule));
      return;
    }

    if (rule.validator) {
      return runValidator(current, rule).then(result => {
        if (result && typeof result === 'string') {
          setValidateFailed(true);
          setValidateMessage(result);
        } else if (result === false) {
          setValidateFailed(true);
          setValidateMessage(getRuleMessage(current, rule));
        }
      });
    }
  }), Promise.resolve());

  const resetValidation = () => {
    if (validateFailed) {
      setValidateFailed(false);
      setValidateMessage('');
    }
  };

  const validate = rules => {
    new Promise(resolve => {
      resetValidation();

      if (rules) {
        runRules(rules).then(() => {
          if (validateFailed) {
            resolve({
              name,
              message: validateMessage
            });
          } else {
            resolve();
          }
        });
      } else {
        resolve();
      }
    });
  };

  const validateWithTrigger = trigger => {
    if (parent && rules) {
      const defaultTrigger = parent.validateTrigger === trigger;
      const results = rules.filter(rule => {
        if (rule.trigger) {
          return rule.trigger === trigger;
        }

        return defaultTrigger;
      });
      validate(results);
    }
  };

  const limitValueLength = value => {
    if (isDef(maxlength) && value.length > maxlength) {
      if (modelValue.length === +maxlength) {
        return modelValue;
      }

      return value.slice(0, +maxlength);
    }

    return value;
  };

  const updateValue = (value, trigger = 'onChange') => {
    value = limitValueLength(value);

    if (type === 'number' || type === 'digit') {
      const isNumber = type === 'number';
      value = formatNumber(value, isNumber, isNumber);
    }

    if (formatter && trigger === formatTrigger) {
      value = formatter(value);
    }

    if (inputRef.current && value !== inputRef.current.value) {
      inputRef.current.value = value;
    }

    if (value !== modelValue) {
      console.log('formatter', value);
      setModelValue(value);
      onUpdateModel == null ? void 0 : onUpdateModel(value);
    }
  };

  const handleInput = event => {
    if (!composing) {
      console.log('event.currentTarget.value', event.currentTarget.value);
      updateValue(event.currentTarget.value);
    }
  };

  const handleBlur = event => {
    setFocused(false);
    updateValue(event.currentTarget.value, 'onBlur');
    onBlur == null ? void 0 : onBlur(event);
    validateWithTrigger('onBlur');
    resetScroll();
  };

  const handleClickInput = event => {
    onClickInput == null ? void 0 : onClickInput(event);
  }; // const handleClickLeftIcon = (event: MouseEvent) => {
  //   onClickLeftIcon?.(event);
  // };


  const handleClickRightIcon = event => {
    onClickRightIcon == null ? void 0 : onClickRightIcon(event);
  };

  const handleClear = event => {
    event.preventDefault();
    setModelValue('');
    onClear == null ? void 0 : onClear(event);
  };

  const showError = () => {
    var _parent$props;

    if (typeof error === 'boolean') {
      return error;
    }

    if (parent != null && (_parent$props = parent.props) != null && _parent$props.showError && validateFailed) {
      return true;
    }

    return false;
  };

  const labelStyle = useMemo(() => {
    const labelWidth = getProp('labelWidth');

    if (labelWidth) {
      return {
        width: addUnit(labelWidth)
      };
    }

    return {
      width: ''
    };
  }, [getProp]);

  const handleCompositionStart = event => {
    if (event.currentTarget) {
      setComposing(true);
    }
  };

  const handleCompositionEnd = event => {
    const {
      currentTarget
    } = event;

    if (composing) {
      // currentTarget.composing = false;
      setComposing(false);
      trigger(currentTarget, 'onInput');
    }
  };

  const adjustSize = () => {
    const input = inputRef.current;

    if (!(type === 'textarea' && autosize) || !input) {
      return;
    }

    input.style.height = 'auto';
    let height = input.scrollHeight;

    if (isObject(autosize)) {
      const {
        maxHeight,
        minHeight
      } = autosize;

      if (maxHeight !== void 0) {
        height = Math.min(height, maxHeight);
      }

      if (minHeight !== void 0) {
        height = Math.max(height, minHeight);
      }
    }

    if (height) {
      input.style.height = `${height}px`;
    }
  }; // const renderLeftIcon = () => {
  //   if (leftIcon) {
  //     return (
  //       <div className={classnames(`${baseClass}__left-icon`])} onClick={handleClickLeftIcon}>
  //         <Icon name={leftIcon} classPrefix={iconPrefix} className={classnames({ [`r-icon`]]: !!leftIcon })} />
  //       </div>
  //     );
  //   }
  // };


  const renderRightIcon = () => {
    if (rightIcon) {
      return /*#__PURE__*/React.createElement("div", {
        className: classnames(`${baseClass}__right-icon`),
        onClick: handleClickRightIcon
      }, /*#__PURE__*/React.createElement(Icon, {
        name: rightIcon,
        classPrefix: iconPrefix,
        className: classnames({
          [`r-icon`]: !!rightIcon
        })
      }));
    }

    return null;
  };

  const renderWordLimit = () => {
    if (showWordLimit && maxlength) {
      const count = getModelValue().length;
      return /*#__PURE__*/React.createElement("div", {
        className: classnames(`${baseClass}__word-limit`)
      }, /*#__PURE__*/React.createElement("span", {
        className: classnames(`${baseClass}__word-num`)
      }, count), "/", maxlength);
    }

    return null;
  };

  const renderMessage = () => {
    if (parent && parent.showErrorMessage === false) {
      return;
    }

    const message = errorMessage || validateMessage;

    if (message) {
      const errorMessageAlign = getProp('errorMessageAlign');
      return /*#__PURE__*/React.createElement("div", {
        className: classnames(`${baseClass}__error-message`, `${baseClass}__error-message--${errorMessageAlign}`)
      }, message);
    }

    return null;
  };

  const formValue = () => {
    if (childFieldValue.current) {
      return childFieldValue.current;
    }

    return modelValue;
  };

  const focus = () => {
    if (inputRef.current) {
      inputRef.current.focus();
    }
  };

  const blur = () => {
    if (inputRef.current) {
      inputRef.current.blur();
    }
  };

  const handleFocus = event => {
    setFocused(true);
    onFocus == null ? void 0 : onFocus(event);
    const readonly = getProp('readonly');

    if (readonly) {
      blur();
    }
  };

  const handleKeypress = event => {
    if (event.key === 'Enter') {
      var _parent$props2;

      const submitOnEnter = parent == null ? void 0 : (_parent$props2 = parent.props) == null ? void 0 : _parent$props2.submitOnEnter;

      if (!submitOnEnter && props.type !== 'textarea') {
        event.preventDefault();
      } // trigger blur after click keyboard search button


      if (type === 'search') {
        blur();
      }
    }

    onKeyPress == null ? void 0 : onKeyPress(event);
  };

  const renderInput = () => {
    const disabled = getProp('disabled');
    const readonly = getProp('readonly');
    const inputAlign = getProp('inputAlign');
    const inputProps = {
      name,
      rows: rows !== undefined ? +rows : undefined,
      className: classnames(`${baseClass}__control`, `${baseClass}__control--${inputAlign}`),
      value: modelValue,
      disabled,
      readOnly: readonly,
      placeholder,
      autoComplete: autocomplete,
      autoFocus: autofocus,
      onBlur: handleBlur,
      onFocus: handleFocus,
      onInput: handleInput,
      onClick: handleClickInput,
      onChange: handleCompositionEnd,
      onKeyPress: handleKeypress,
      onCompositionStart: handleCompositionStart,
      onCompositionEnd: handleCompositionEnd
    };
    let inputType = type;
    let mode;

    if (type === 'textarea') {
      return /*#__PURE__*/React.createElement("textarea", _extends({
        ref: inputRef
      }, inputProps));
    } // type="number" is weired in iOS, and can't prevent dot in Android
    // so use inputmode to set keyboard in mordern browers


    if (type === 'number') {
      inputType = 'text';
      mode = 'decimal';
    }

    if (type === 'digit') {
      inputType = 'tel';
      mode = 'numeric';
    }

    return /*#__PURE__*/React.createElement("input", _extends({
      type: inputType,
      ref: inputRef,
      inputMode: mode
    }, inputProps));
  };

  useImperativeHandle(ref, () => ({
    focus,
    blur,
    formValue,
    validate,
    resetValidation
  }));
  useEffect(() => {
    resetValidation();
    validateWithTrigger('onChange');
    adjustSize();
  }, [modelValue]);
  return /*#__PURE__*/React.createElement(Cell, {
    ref: ref,
    title: label,
    size: size,
    icon: leftIcon,
    className: classnames(baseClass, {
      [`${baseClass}--error`]: showError(),
      [`${baseClass}--disabled`]: disabled,
      [`${baseClass}__label--${labelAlign}`]: !!labelAlign,
      [`${baseClass}--min-height}`]: type === 'textarea' && !autosize
    }),
    center: center,
    border: border,
    isLink: isLink,
    required: required,
    clickable: clickable,
    titleStyle: labelStyle,
    valueClass: classnames(`${baseClass}__value`),
    titleClass: classnames(`${baseClass}__label`, `${baseClass}__label--${labelAlign}`, labelClass),
    arrowDirection: arrowDirection
  }, /*#__PURE__*/React.createElement("div", {
    className: classnames(`${baseClass}__body`)
  }, renderInput(), showClear && /*#__PURE__*/React.createElement(Icon, {
    name: "clear",
    className: classnames(`${baseClass}__clear`),
    onClick: handleClear
  }), renderRightIcon(), React.Children.map(children, child => {
    var _child$type;

    if ((child == null ? void 0 : (_child$type = child.type) == null ? void 0 : _child$type.name) === 'Button') {
      return /*#__PURE__*/React.createElement("div", {
        className: classnames(`${baseClass}__button`)
      }, children);
    }

    return null;
  })), renderWordLimit(), renderMessage());
});
Field.displayName = 'Field';

export default Field;
export { useRefCallback };
